"{ Package: 'stx:goodies/xtreams/core/tests' }"

"{ NameSpace: Xtreams }"

TestCase subclass:#ReadingWritingTest
	instanceVariableNames:'output input'
	classVariableNames:''
	poolDictionaries:'XtreamsPool'
	category:'Xtreams-Core-Tests'
!

ReadingWritingTest comment:'Tests in this hierarchy have to be written so that they run with all kinds of terminals. Consequently they generally need to do all their writing first and then reading, so that we can run against collection streams without the assumption that the collection preserves identity as it''s being grown by the write stream. Also the streams should be accessed via accessors for the same reasons.

Instance Variables
	output  <Object> undocumented
	input   <Object> undocumented

'
!


!ReadingWritingTest class methodsFor:'testing'!

isAbstract
    "Override to true if a TestCase subclass is Abstract and should not have
     TestCase instances built from it"

    ^ self subclasses notEmpty
! !

!ReadingWritingTest methodsFor:'accessing'!

input

	^input
!

output

	^output
! !

!ReadingWritingTest methodsFor:'tests - core'!

testInsert
	self output write: #[ 1 2 3 4 7 8 9 0 ].
	[self output -- 4] on: Error do: [self deny: self output isPositionable].
	self assert: (self output insert: #[ 5 6 ]) = 2.
	self input isPositionable
		ifTrue: [self assert: (self input read: 10) = #[ 1 2 3 4 5 6 7 8 9 0 ]]
		ifFalse:        [self assert: (self input read: 10) = #[ 1 2 3 4 7 8 9 0 5 6 ]]
!

testPut

	self output put: 1.
	self assert: self input get = 1
!

testReadIntoCollection

	| data out |
	data := #[1 2 3 4 5 6 7].
	self output write: data; close.
	out := ByteArray new: data size.
	self assert: (self input read: 4 into: out) = 4.
	self assert: (self input read: 3 into: out at: 5)  = 3.
	self assert: out = data
!

testReadWriteLargeAmount
	| data |
	1 to: 10 do: [:each | self output put: 1. self assert: self input get = 1].
	#( 64 1024 2048 4096 8192 ) do: [:dataSize |
		data := ByteArray new: dataSize.
		1 to: data size do: [:i | data at: i put: (i - 1) \\ 256].
		self    timeout:        1000 milliseconds
			server: [1 to: 10 do: [:each | self output write: data]. true]
			client:         [(1 to: 10) inject: true into: [:pass :each | pass and: [(self input read: data size) = data]]]]
!

testWriteCollection

	| data |
	data := #[1 2 3 4 5 6 7].
	self assert: (self output write: data) = data size.
	self assert: (self input read: data size) = data
!

testWriteFromCollectionAt

	| data |
	data := #[1 2 3 4 5 6 7].
	self assert: (self output write: 4 from: data at: 1) = 4.
	self assert: (self input read: 4) = (data first: 4).
	self assert: (self output write: 3 from: data at: 5) = 3.
	self assert: (self input read: 3) = (data last: 3)
!

testWriteFromStream
	| data |
	data := #[ 1 2 3 4 ] reading.
	self assert: (self output write: 3 from: data) = 3.
	self assert: (self input read: 3) = #[ 1 2 3 ].
	self assert: (self output write: 1 from: data) = 1.
	self assert: self input get = 4.
	self assert: (
		[       self output write: 2 from: data. false
		] on: Incomplete do: [ :eos |
			eos count = 0 ])
! !

!ReadingWritingTest methodsFor:'tests - core - positioning'!

testReadPositioningSeek

	| record in |
	record := #[ 1 2 3 4 5 6 7 8 9 0 ].
	self output write: record; close.
	in := self input positioning.
	self assert: (in ++ 7) = 7.
	self assert: in position = 7.
	self assert: (in -- 3) = 3.
	self assert: in position = 4.
!

testWriteExplore

	| record mark out |
	record := #[ 1 2 3 4 5 6 7 8 9 0 ].
	out := self output positioning.
	mark := out position.
	out
		write: 0;
		write: record;
		explore: [out += mark; write: record size];
		close.
	self assert: (self input read: record size + 1) = ((ByteArray with: record size), record)
!

testWritePositioningSeek

	| record out |
	record := #[ 1 2 3 4 5 6 7 8 9 0 ].
	out := self output positioning.
	out     write: record.
	self assert: (out -- 7) = 7.
	self assert: out position = 3.
	self assert: (out ++ 3) = 3.
	self assert: out position = 6.
	self assert: (out ++ 4) = 4.
	out close.
	self assert: (self input read: record size) = record
! !




!ReadingWritingTest methodsFor:'utilities'!

timeout: timeoutDuration server: serverBlock client: clientBlock
	"For tests that must be able to write and read at the same time, where they may block on each.
	 Sockets, Pipes have an operating system buffer which dictates its bandwidth."

	self assert: serverBlock value.
	self assert: clientBlock value.
! !

!ReadingWritingTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

