"{ Package: 'stx:goodies/xtreams/core/tests' }"

"{ NameSpace: Xtreams }"

TestCase subclass:#RingBufferTest
	instanceVariableNames:'output buffer'
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Core-Tests'
!


!RingBufferTest methodsFor:'initialize-release'!

setUp

	buffer := RingBuffer on: 'xxxx' copy
! !

!RingBufferTest methodsFor:'tests'!

testClear

	| cacheSize |
	buffer write: 4 from: '1234' at: 1.
	cacheSize := buffer cacheSize.
	buffer clear.
	self assert: buffer cacheSize = cacheSize.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer writeSize = cacheSize
!

testInsertInTheMiddle

	| chunk |
	chunk := 'hello'.
	buffer := RingBuffer on: (String new: chunk size * 5).
	buffer write: chunk; write: chunk.
	buffer writePosition: chunk size.
	buffer insert: chunk.
	self assert: buffer writePosition = (chunk size * 2).
	self assert: buffer activeSize = (chunk size * 3)
!

testRead

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 1 into: output at: 1.
	self assert: output = '1www'.
	self assert: buffer readPosition = 1.
	self assert: buffer readSize = 3.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 1.
	buffer read: 3 into: output at: 2.
	self assert: output = '1234'.
	self assert: buffer readPosition = 4.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 4
!

testReadOverflow

	output := 'wwwwww' copy.
	buffer write: 4 from: '1234' at: 1.
	self should: [buffer read: 6 into: output at: 1] raise: Incomplete.
	self assert: output = '1234ww'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 4
!

testReadWrap

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 3 into: output at: 1.
	buffer write: 3 from: 'ABC' at: 1.
	output := 'wwww' copy.
	buffer read: 4 into: output at: 1.
	self assert: output = '4ABC'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 4
!

testReadWriteFirstHalf

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 2 into: output at: 1.
	self assert: output = '12ww'.
	self assert: buffer cache = '1234'.
	self assert: buffer readPosition = 2.
	self assert: buffer readSize = 2.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 2.
	buffer write: 2 from: 'AB' at: 1.
	self assert: buffer cache = 'AB34'.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 4.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 0.
	output := 'wwww' copy.
	buffer read: 2 into: output at: 1.
	self assert: output = '34ww'.
	self assert: buffer readPosition = 2.
	self assert: buffer readSize = 2.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 2
!

testReadWriteSecondHalf
	" <--- SomeData ... ReadAt ... WriteAt/DataEnd DataStart ... SomeData ---> "

	output := 'wwww' copy.
	buffer write: 3 from: '123' at: 1.
	buffer read: 3 into: output at: 1.
	buffer write: 3 from: 'ABC' at: 1.
	buffer read: 2 into: output at: 1.
	output := 'wwww' copy.
	buffer read: 1 into: output at: 1.
	self assert: output = 'Cwww'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 4
!

testReadWriteWrap

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 3 into: output at: 1.
	self assert: output = '123w'.
	self assert: buffer cache = '1234'.
	self assert: buffer readPosition = 3.
	self assert: buffer readSize = 1.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 3.
	buffer write: 1 from: 'A' at: 1.
	self assert: buffer cache = 'A234'.
	self assert: buffer readPosition = 2.
	self assert: buffer readSize = 2.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 2.
	buffer write: 2 from: 'BC' at: 1.
	self assert: buffer cache = 'ABC4'.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 4.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 0.
	output := 'wwww' copy.
	buffer read: 4 into: output at: 1.
	self assert: output = '4ABC'.
	self assert: buffer readPosition = 4.
	self assert: buffer readSize = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer writeSize = 4
!

testTrim

	output := 'wwww' copy.
	buffer write: 3 from: '1234' at: 1.
	buffer read: 1 into: output at: 1.
	buffer writeSkip: -1.
	buffer trim.
	self assert: buffer cacheSize = 2.
	self assert: buffer cache = '23'.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 2.
	self assert: buffer writePosition = 2.
	self assert: buffer writeSize = 0
!

testTrimInsideOut

	output := 'wwww' copy.
	buffer write: 4 from: '1234' at: 1.
	buffer read: 3 into: output at: 1.
	buffer write: 2 from: 'AB' at: 1.
	buffer writeSkip: -1.
	buffer trim.
	self assert: buffer cacheSize = 2.
	self assert: buffer cache = '4B'.
	self assert: buffer readPosition = 0.
	self assert: buffer readSize = 2.
	self assert: buffer writePosition = 2.
	self assert: buffer writeSize = 0
!

testWrite

	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 4.
	buffer write: 1 from: '1234' at: 1.
	self assert: buffer cache = '1xxx'.
	self assert: buffer activeSize = 1.
	self assert: buffer inactiveSize = 3.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 1.
	self assert: buffer readSize = 1.
	self assert: buffer writeSize = 3.
	buffer write: 3 from: '1234' at: 2.
	self assert: buffer cache = '1234'.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0
!

testWriteObject
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 0.
	self assert: buffer inactiveSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 0.
	self assert: buffer readSize = 0.
	self assert: buffer writeSize = 4.

	buffer put: $1.
	self assert: buffer cache first = $1.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 1.
	self assert: buffer inactiveSize = 3.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 1.
	self assert: buffer readSize = 1.
	self assert: buffer writeSize = 3.
	buffer put: $2; put: $3; put: $4.
	self assert: buffer cache = '1234'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 4.
	self assert: buffer inactiveSize = 0.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0.
	buffer put: $5.
	self assert: buffer cache = '5234'.
	self assert: buffer cacheSize = 4.
	self assert: buffer activeSize = 4.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
!

testWriteOverflow0

	buffer write: 1 from: '1234' at: 1.
	buffer write: 4 from: 'ABCD' at: 1.
	self assert: buffer cache = 'ABCD'.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0
!

testWriteOverflow1

	buffer write: 1 from: '1234' at: 1.
	buffer write: 5 from: 'ABCDE' at: 1.
	self assert: buffer cache = 'BCDE'.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0
!

testWriteWrap

	buffer write: 3 from: '1234' at: 1.
	buffer write: 2 from: 'AB' at: 1.
	self assert: buffer cache = 'B23A'.
	self assert: buffer readPosition = 0.
	self assert: buffer writePosition = 4.
	self assert: buffer readSize = 4.
	self assert: buffer writeSize = 0
! !

!RingBufferTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

