"{ Package: 'stx:goodies/xtreams/transforms/tests' }"

"{ NameSpace: Xtreams }"

Object subclass:#ObjectMarshalerTest
	instanceVariableNames:'stream marshaling log bytes marshaler'
	classVariableNames:''
	poolDictionaries:''
	category:'Xtreams-Transforms-Tests'
!


!ObjectMarshalerTest methodsFor:'initialize-release'!

setUp
	marshaler := ObjectMarshaler new.
	stream := ByteArray new writing.
	marshaling := stream marshaling: marshaler.
	bytes := nil.
	log := nil
! !

!ObjectMarshalerTest methodsFor:'private'!

analyse
	"Run an analysis on the stream contents to ensure it has integrity"
	bytes := stream conclusion.
	log := String new writing.
	(ObjectAnalyseStream on: bytes reading marshaler: marshaler class new) do: [:each | log write: each].
	log := log conclusion.
	marshaling := bytes reading marshaling: marshaler class new
! !

!ObjectMarshalerTest methodsFor:'tests'!

testArray
	marshaling put: (Array with: 1 with: -2.7 with: true).
	self analyse.
	self assert: marshaling get = (Array with: 1 with: -2.7 with: true)
!

testBlockClosureClean
	marshaling put: [].
	marshaling put: [:x :y | x + y].
	self analyse.
	self assert: marshaling get value = nil.
	self assert: (marshaling get value: 2 value: 3) = 5
!

testBlockClosureDirty
	| a a_original a_copy |
	a := 1.
	a_original := [a].
	marshaling put: a_original.
	self analyse.
	a_copy := marshaling get.
	self assert: a_original value = 1.
	self assert: a_copy value = 1.
	a := 2.
	self assert: a_original value = 2.
	self assert: a_copy value = 1
!

testCharacters
	marshaling write: 'testing'.
	self analyse.
	self assert: marshaling rest = #( $t $e $s $t $i $n $g )
!

testClass
	marshaling put: UndefinedObject.
	marshaling put: ObjectMarshalerTest.
	self analyse.
	self assert: marshaling get = UndefinedObject.
	self assert: marshaling get = ObjectMarshalerTest
!

testDictionary
	| result |
	marshaling put: (Dictionary new at: 1 put: 'one'; at: #two put: 2; at: 'three' put: (4 @ 5); yourself).
	self analyse.
	result := marshaling get.
	self assert: (result at: 1) = 'one'.
	self assert: (result at: #two) = 2.
	self assert: (result at: 'three') = (4 @ 5)
!

testFloat
	marshaling put: 5.4.
	marshaling put: -172.11102.
	self analyse.
	self assert: marshaling get = 5.4.
	self assert: marshaling get = -172.11102
!

testInteger
	marshaling put: 0.
	marshaling put: 1.
	marshaling put: 255.
	marshaling put: 254.
	marshaling put: -1.
	marshaling put: SmallInteger maxVal.
	marshaling put: SmallInteger minVal.
	self analyse.
	self assert: marshaling get = 0.
	self assert: marshaling get = 1.
	self assert: marshaling get = 255.
	self assert: marshaling get = 254.
	self assert: marshaling get = -1.
	self assert: marshaling get = SmallInteger maxVal.
	self assert: marshaling get = SmallInteger minVal.
!

testInterval
	marshaling put: (1 to: 12 by: 2).
	self analyse.
	self assert: marshaling get = (1 to: 11 by: 2)
!

testLargeInteger
	marshaling put: SmallInteger maxVal * 2.
	marshaling put: SmallInteger minVal * 2.
	marshaling put: 2 ** 256.
	marshaling put: -2 ** 256.
	self analyse.
	self assert: marshaling get = (SmallInteger maxVal * 2).
	self assert: marshaling get = (SmallInteger minVal * 2).
	self assert: marshaling get = (2 ** 256).
	self assert: marshaling get = (-2 ** 256)
!

testMissingClass
	| missingClass missingInstance object |
	missingClass := ObjectMarshaler basicNew
		resolveClass: 'fulluble'
		isMeta: false
		format: (Behavior formatFromType: #none super: Object instVars: #('a' 'b' 'c'))
		instVarNames: #('a' 'b' 'c').
	missingInstance := missingClass new.
	missingInstance instVarNamed: 'a' put: 1.
	missingInstance instVarNamed: 'b' put: 2.
	missingInstance instVarNamed: 'c' put: 3.
	marshaling put: missingInstance.
	self analyse.
	object := marshaling get.
	self assert: object class ~~ missingClass.
	self assert: object class superclass = Object.
	self assert: (object instVarNamed: 'a') = 1.
	self assert: (object instVarNamed: 'b') = 2.
	self assert: (object instVarNamed: 'c') = 3.
	self assert: object a = 1.
	self assert: object b = 2.
	self assert: object c = 3.
	object a: 4.
	object b: 5.
	object c: 6.
	self assert: object a = 4.
	self assert: object b = 5.
	self assert: object c = 6.
!

testNamespace
	marshaling put: Root.Smalltalk.
	marshaling put: Root.Smalltalk.Xtreams.
	self analyse.
	self assert: marshaling get = Root.Smalltalk.
	self assert: marshaling get = Root.Smalltalk.Xtreams
!

testOrderedCollection
	| result |
	marshaling put: (OrderedCollection new add: #test; add: 'test'; add: 5; yourself).
	self analyse.
	result := marshaling get.
	self assert: result first = #test.
	self assert: (result at: 2) = 'test'.
	self assert: result last = 5
!

testPoint
	marshaling put: -1 @ 2.
	marshaling put: 3.4 @ -4.5.
	marshaling put: 5 @ 5.
	self analyse.
	self assert: marshaling get = (-1 @ 2).
	self assert: marshaling get = (3.4 @ -4.5).
	self assert: marshaling get = (5 @ 5)
!

testRecursion
	| value result |
	value := nil asValue.
	value value: value.
	marshaling put: value.
	self analyse.
	result := marshaling get.
	self assert: result value == result
!

testSortedCollection
	| sortBlock result |
	sortBlock := Kernel.Compiler evaluate: '[:a :b | a <= b]'.
	marshaling put: ((SortedCollection sortBlock: sortBlock) add: 5; add: 4; add: 1; add: 3; add: 2; yourself).
	self analyse.
	result := marshaling get.
	self assert: result asArray = #(1 2 3 4 5).
	result add: 1.5.
	self assert: result asArray = #(1 1.5 2 3 4 5)
!

testString
	marshaling put: 'testing Î©'.
	self analyse.
	self assert: marshaling get = 'testing Î©'
!

testTranscript
	marshaling put: Transcript.
	marshaling put: (Array with: Transcript).
	self analyse.
	self assert: marshaling get == Transcript.
	self assert: marshaling get = (Array with: Transcript)
! !

!ObjectMarshalerTest class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

